@@ -1,5 +1,5 @@
 /************************************************************************/
-/* Copyright (C) 2009-2011 Lieven Moors and Jari Suominen               */
+/* Copyright (C) 2009-2012 Lieven Moors and Jari Suominen               */
 /*                                                                      */
 /* This file is part of aj-snapshot.                                    */
 /*                                                                      */
@@ -22,15 +22,18 @@
 extern int verbose;
 extern int daemon_running;
 extern int jack_dirty;
-extern pthread_mutex_t callback_lock;
+extern pthread_mutex_t registration_callback_lock;
 extern pthread_mutex_t shutdown_callback_lock;
-int restore_successful;
+extern int jack_success;
 
-/* Callbacks */
-int jack_graph_order (void *arg) {
-    pthread_mutex_lock( &callback_lock );
-    jack_dirty++;
-    pthread_mutex_unlock( &callback_lock );
+
+void jack_port_registration (jack_port_id_t port, int reg, void *arg) {
+    /* This callback is called when new ports appear. */
+    if(reg){
+        pthread_mutex_lock( &registration_callback_lock );
+        jack_dirty++;
+        pthread_mutex_unlock( &registration_callback_lock );
+    }
 }
 
 void jack_shutdown (void * jackc) {
@@ -48,6 +51,7 @@ void jack_shutdown (void * jackc) {
 
 void jack_initialize( jack_client_t** jackc, int callbacks_on )
 {
+    int result;
     jack_options_t options = JackNoStartServer;
     *jackc = jack_client_open("aj-snapshot", options, NULL);
 
@@ -59,17 +63,21 @@ void jack_initialize( jack_client_t** jackc, int callbacks_on )
     jack_on_shutdown(*jackc, jack_shutdown, jackc);
 
     if (callbacks_on) {
-        jack_set_graph_order_callback(*jackc, jack_graph_order, 0);
-        
+        result = jack_set_port_registration_callback(*jackc, jack_port_registration, 0);
+        if(result < 0) fprintf(stderr, "aj-snapshot: Failed to set the port registration callback.\n");
         if (jack_activate (*jackc)) {
-            fprintf (stderr, "aj-snapshot: Jack server seems to be running but is not responding.");
+            fprintf (stderr, "aj-snapshot: Jack server seems to be running but is not responding.\n");
             jack_client_close(*jackc); 
             *jackc = NULL;
             return;
         }
     }
     
-    if (verbose && daemon_running) fprintf(stderr, "aj-snapshot: Jack server was started.\n");
+    pthread_mutex_lock( &registration_callback_lock );
+    jack_dirty = 1;
+    pthread_mutex_unlock( &registration_callback_lock );
+
+    if (verbose && daemon_running) fprintf(stdout, "aj-snapshot: Jack server was started.\n");
 
     return;
 }
@@ -90,7 +98,7 @@ void jack_restore_connections( jack_client_t** jackc, const char* client_name, c
     while (connection_node){
         dest_port = mxmlElementGetAttr(connection_node, "port");
 
-        strcpy(tmp_str, dest_port); // otherwise we change the names of ports in jack !!
+        strcpy(tmp_str, dest_port); // dest_port is 'const char*'
         dest_client_name = strtok(tmp_str, ":");
 
         if(!is_ignored_client(dest_client_name)){
@@ -104,11 +112,6 @@ void jack_restore_connections( jack_client_t** jackc, const char* client_name, c
             }
             pthread_mutex_unlock( &shutdown_callback_lock );
 
-            if (err == 0) {
-                pthread_mutex_lock( &callback_lock );
-                jack_dirty--;
-                pthread_mutex_unlock( &callback_lock );
-            }
             if(verbose){
                 if (err == 0) {
                     fprintf(stdout, "Connecting port '%s' with '%s'\n", src_port, dest_port);
@@ -119,7 +122,7 @@ void jack_restore_connections( jack_client_t** jackc, const char* client_name, c
                     } 
                     else {
                         fprintf(stdout, "Failed to connect port '%s' to '%s'!\n", src_port, dest_port);
-                        restore_successful = 0;
+                        jack_success = 0;
                     }
                 }
             }
@@ -207,8 +210,8 @@ void jack_store_connections( jack_client_t* jackc, const char* port_name, mxml_n
 void jack_store( jack_client_t* jackc, mxml_node_t* xml_node )
 {
     mxml_node_t* jack_node = mxmlNewElement(xml_node, "jack");
-    mxml_node_t* client_node;
-    mxml_node_t* port_node;
+    mxml_node_t* client_node = NULL;
+    mxml_node_t* port_node = NULL;
 
     char tmp_str[jack_port_name_size()];
     char client_name_prev[jack_port_name_size()];
